package javaspacesmandelbrot;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;

import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;

/**
 * Title:        Mandelbrot Set als JavaSpaces Anwendung
 * Description:  Einfaches Beispiel fr den Master-Slave Aufbau einer JavaSpaces Anwendung
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

public class Master extends Applet
    implements Runnable, MouseListener, MouseMotionListener
{
    private JavaSpace space;
    private int xsize;      // Window Grsse
    private int ysize;
    private Long jobId;     // ID fr die Berechnung
    private int tasks;      // Anzahl Tasks, die generiert werden sollen
    private int lines = 0;  // Anzahl Zeilen pro Task
    private Thread master;

    // Mandelbrot Region / Parameter
    private double x1 = -2.25;
    private double x2 =  3.0;
    private double y1 = -1.8;
    private double y2 =  3.3;

    private boolean done = false;   // ist die Berechnung beendet?
    private int progress;           // Anzahl Mandelbrot Zeilen
    private boolean drag = false;   // Zoom?

    // Screen Buffer und Graphics
    private Image offscreen;
    private Graphics offg;

    public void init() {
        space = SpaceAccessor.getSpace();
        xsize = getSize().width;
        ysize = getSize().height;

        String taskStr = getParameter("Task");
        if (taskStr == null) {
            tasks = 20;
        } else {
            tasks = Integer.parseInt(taskStr);
        }
        lines = ysize / tasks;  // Abzahl Mandelbrotzeilen pro Task

        // Buffer
        offscreen = createImage(xsize, ysize);
        offg = offscreen.getGraphics();
        offg.setColor(Color.black);
        offg.fillRect(0, 0, xsize, ysize);

        // listeners
        this.addMouseListener(this);
        this.addMouseMotionListener(this);

        // Threads fr die Berechnungen
        if (master == null) {
            master = new Thread(this);
            master.start();
        }
    }

    public void run() {
        generateTasks();
        collectResults();
    }

    private void generateTasks() {
        Task task = new Task();

        long millis = System.currentTimeMillis();
        jobId = new Long(millis);
        task.jobId = jobId;

        task.x1 = new Double(x1);
        task.x2 = new Double(x2);
        task.y1 = new Double(y1);
        task.y2 = new Double(y2);

        task.width = new Integer(xsize);
        task.height = new Integer(ysize);
        task.lines = new Integer(lines);

        System.out.println("[Mandelbrot.Master]Verteilen der Arbeit");
        for (int i = 0; i < ysize; i += lines) {
            task.start = new Integer(i);
            try {
                System.out.println("[Mandelbrot.Master]Task " +
                    task.start + " fuer Job " + task.jobId);
                space.write(task, null, Lease.FOREVER);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void collectResults() {
        while (true) {
            while (done) {
                try {
                    System.out.println(
                        "[Mandelbrot.Master]warten auf neue Aufgabe.");
                    Thread.sleep(500);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }

            Result template = new Result();
            template.jobId = jobId;
            Result result = null;

            try {
                result = (Result)
                    space.take(template, null, Long.MAX_VALUE);
                System.out.println("[Mandelbrot.Master]Einsammeln der Ergebnisse " +
                    result.start + " fuer Job " + result.jobId);
                display(result);
                progress = progress + lines;
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (progress == ysize) {
                done = true;
                repaint();
            }
        }
    }

    public void mouseDragged(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        if (done) {
            drag=true;
            Graphics g=this.getGraphics();
            g.drawImage(offscreen,0,0,this);
            g.setColor(Color.white);
            g.drawRect(x-xsize/4,y-ysize/4,xsize/2,ysize/2);
        }
    }

    public void mouseReleased(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        if (done) {
            x1 += ((float)x / (float)xsize) * x2;
            y1 += ((float)y / (float)ysize) * y2;
            x2 /= 2.0;
            y2 /= 2.0;
            x1 -= x2 / 2.0;
            y1 -= y2 / 2.0;
            done = false;
            drag = false;
            offg.setColor(Color.black);
            offg.fillRect(0, 0, xsize, ysize);
            progress = 0;
            repaint();
            generateTasks();
        }
    }

    public void update( Graphics g ) {
        if (!drag) {
                paint(g);
        }
    }

    public void paint( Graphics g ) {
        if (!drag) {
            if (done) {
                g.drawImage(offscreen,0,0,this);
            }
            else {
                g.drawImage(offscreen,0,0,this);
                g.setColor(Color.white);
                g.drawRect(xsize/4, 10, xsize/2, 5);
                g.fillRect(xsize/4, 11, (progress*(xsize/2))/ysize, 4);
            }
        }
    }

    private Color getPixelColor(int val) {
        Color color;

        if (val == 100)                 color = new Color(0,0,0);
        else if (val > 90)              color = new Color(val*2,0,(val-90)*25);
        else if (val > 80)              color = new Color(val*2,0,0);
        else if (val > 60)              color = new Color(val*3,0,val);
        else if (val > 20)              color = new Color(val*4,0,val*2);
        else if (val > 10)              color = new Color(val*5,0,val*10);
        else                                color = new Color(0,0,val*20);
        return color;
    }

    private void display(Result result) {
        int j = 0;
        for(int l=result.start.intValue();j < lines; j++, l++) {
            for(int k = 0; k < xsize; k++) {
                int n = result.points[k][j];
                Color pixelColor = getPixelColor(result.points[k][j]);
                offg.setColor(pixelColor);
                offg.fillRect(k, l, 1, 1);
            }
        }
        repaint();
    }

    public void mousePressed(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}

}
