package javaspacessynchronisation;

import java.io.*;
import java.rmi.*;
import java.util.Hashtable;

import net.jini.core.lease.Lease;
import net.jini.core.lookup.*;
import net.jini.discovery.*;
import net.jini.space.JavaSpace;

/**
 * Title:        Synchronisation von JavaSpaces Anwendungen
 * Description:  Einfaches Spiel, welches die Synchronisation mehrerer Tasks zeigt, welche auf Objekte in einem Java Space zugreifen.
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

// Implementation des Game Service: publiziert ein Proxy
// ber den die Clients sich an das Spiel anschliessen knnen


// Proxy Objekt und Implementation des GameServiceInterface

class GameServiceProxy implements Serializable, GameServiceInterface {
    private JavaSpace space;
    private String name;
    private Game game;

    public GameServiceProxy() {
    }

    public void setSpace(JavaSpace space) {
        this.space = space;
    }

    // hole ein Ticket aus dem JavaSpace, um am Spiel teilzunehmen
    public Game joinGame(String name) {
        this.name = name;
        System.out.println("Ein Ticket bitte " + name);

        Ticket ticketTemplate = new Ticket(name);
        Ticket ticket;
        try {
            ticket = (Ticket)space.take(ticketTemplate, null, Long.MAX_VALUE);
            this.game = ticket.game;
            System.out.println("Ich habe ein Ticket fuer " + name);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return game;
    }

    // beim verlassen wird das Ticket an das Space zurckgegeben
    public void leaveGame() {
        if (game == null) {
            System.out.println("Ich bin nicht im Spiel!");
        } else {
            Ticket ticketEntry = new Ticket(name, game);
            try {
                space.write(ticketEntry, null, Lease.FOREVER);
            } catch (Exception e) {
                e.printStackTrace();
            }
            System.out.println("Ich verlasse das Spiel " + name);
            name = null;
            game = null;
        }
    }
}


// Jini Teil:
// GameService findet Lookup Service und publiziert das Spiel

public class GameService implements Runnable {
    protected JavaSpace space;
    protected GameImpl game;
    protected final int LEASE_TIME = 30 * 60 * 1000;    // 30 min. leases
    protected ServiceItem item;
    protected Hashtable registrations = new Hashtable();
    protected LookupDiscovery lookupDiscovery;


    // Innere Klasse
    //
    class DiscListener implements DiscoveryListener {

        // neuer Lookup Dienst wurde gefunden
        public void discovered(DiscoveryEvent event) {
            System.out.println("Ein neuer Lookup Service wurde gefunden");
            ServiceRegistrar[] newRegistrars = event.getRegistrars();
            for (int i = 0; i < newRegistrars.length; i++) {
                if (!registrations.containsKey(newRegistrars[i])) {
                    registerWithLookup(newRegistrars[i]);
                }
            }
        }

        // ein Lookup Dienst wird ignoriert / discarded
        public void discarded(DiscoveryEvent event) {
            ServiceRegistrar[] discardedRegistrars = event.getRegistrars();
            for (int i = 0; i < discardedRegistrars.length; i++) {
                registrations.remove(discardedRegistrars[i]);
            }
        }
    }

    protected synchronized void registerWithLookup(ServiceRegistrar registrar) {
        ServiceRegistration registration = null;

        try {
            registration = registrar.register(item, LEASE_TIME);
        } catch (RemoteException ex) {
            System.out.println("Fehler beim Registreren: " + ex.getMessage());
            return;
        }

        // if first registration of the GameService, use the service ID
        if (item.serviceID == null) {
            item.serviceID = registration.getServiceID();
            System.out.println("serviceID : " + item.serviceID);
        }

        registrations.put(registrar, registration);
    }


    // Methode, welche die Tickets ins Space eintrgt
    private void createTickets(String name, int maxPlayers) {

        // initialisieren des Spiels
        try {
            game = new GameImpl(name);
        } catch (RemoteException e) {
            e.printStackTrace();
        }

        // maxPlayers : Anzahl max Players, ans Space bergeben
        for (int i = 0; i < maxPlayers; i++) {
            Ticket ticket = new Ticket(name, game);
            try {
                space.write(ticket, null, Lease.FOREVER);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    // Methode kreiert Proxy, der das GameServiceInterface imlementiert
    protected GameServiceInterface createProxy() {
        GameServiceInterface gameServiceProxy = new GameServiceProxy();
        gameServiceProxy.setSpace(space);
        return gameServiceProxy;
    }

    public GameService(String gameName, int numTickets) throws IOException {
        space = SpaceAccessor.getSpace();

        item = new ServiceItem(null, createProxy(), null);

        // SecurityManager setzen
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        // "pubblic" Klasse (leere Zeichenkette)
        lookupDiscovery = new LookupDiscovery(new String[] {""});

        // Listener soll neue Lookup Services finden
        lookupDiscovery.addDiscoveryListener(new DiscListener());

        // kreiere Tickets fr das Game
        createTickets(gameName, numTickets);
    }

    // schlafender Thread: JVM soll noch aktiv bleiben
    public void run() {
        while (true) {
            try {
                Thread.sleep(1000000);
            } catch (InterruptedException ex) {
            }
        }
    }

    // Main kreiert GameService und startet ihn
    public static void main(String args[]) {
        if (args.length < 2) {
            System.out.println("Usage: GameService gameName numTickets");
            System.exit(1);
        }

        try {
            System.out.println("[GameService]Start");
            String gameName = args[0];
            int numPlayers = Integer.parseInt(args[1]);
            System.out.println("[GameService]Spiel : "+args[0]+" max. Anzahl Spieler : "+args[1]);
            GameService gameService = new GameService(gameName, numPlayers);
            new Thread(gameService).start();
            System.out.println("[GameService]Ende");
        } catch (Exception e) {
            System.out.println("[GameService]konnte nicht kreiert werden: " + e.getMessage());
            e.printStackTrace();
        }
    }

}