package javaspacessynchronisation;

import java.io.IOException;
import java.rmi.*;
import java.util.Vector;

import net.jini.core.lookup.*;
import net.jini.discovery.*;

/**
 * Title:        Synchronisation von JavaSpaces Anwendungen
 * Description:  Einfaches Spiel, welches die Synchronisation mehrerer Tasks zeigt, welche auf Objekte in einem Java Space zugreifen.
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

// Player : spielt mit im GameSpace

public class Player implements Runnable {
    protected String gameName;
    protected String myName;

    protected ServiceTemplate serviceTemplate;
    protected LookupDiscovery discovery;

    // Finde Discovery Events
    class Listener implements DiscoveryListener {
        public void discovered(DiscoveryEvent event) {
            System.out.println("[DiscoveryListener<-Player]discovered()");
            ServiceRegistrar[] newRegistrars = event.getRegistrars();
            for (int i = 0; i < newRegistrars.length ; i++) {
                lookForService(newRegistrars[i]);
            }
        }
        public void discarded(DiscoveryEvent event) {
            System.out.println("[DiscoveryListener<-Player]discarded()");
        }
    }

    public Player(String gameName, String myName) throws IOException {
        System.out.println("[Player]Player("+gameName+","+myName+")");
        this.gameName = gameName;
        this.myName = myName;

        Class[] types = { GameServiceInterface.class };
        serviceTemplate = new ServiceTemplate(null, types, null);

        // security manager
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        // nur public durchsuchen ("")
        discovery = new LookupDiscovery(new String[] {""});

        // Listener
        discovery.addDiscoveryListener(new Listener());
    }

    // Neuer Lookup Dienst: untersuche Proxy
    // welcher GameServiceInterface implementiert
    protected void lookForService(ServiceRegistrar lookupService) {
        System.out.println("[Player]lookForService("+lookupService+")");
        Object proxy = null;

        try {
            proxy = lookupService.lookup(serviceTemplate);
        } catch (RemoteException e) {
            System.err.println("Fehler beim Lookup: " + e.getMessage());
            return;
        }

        if (proxy == null) {
            System.err.println("Service nicht gefunden.");
            return;
        }

        System.out.println("Es wurde ein passender Service gefunden.");

        GameServiceInterface gameInterface = (GameServiceInterface)proxy;

        while (true) {
            Game game = gameInterface.joinGame(gameName);
            if (game != null) {
                try {
                    System.out.println("Spieler ist aktiv: " + gameName);
                    game.play(myName);
                } catch (RemoteException e) {
                    e.printStackTrace();
                }
                gameInterface.leaveGame();
            } else {
                System.out.println("Konnte nicht am Spiel teilnehmen: " + gameName);
            }

            // schlafe 10 sec und versuch's ernneut
            try {
                Thread.sleep(10000);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    // Discovery: beschftige JVM, damit sie aktiv bleibt
    public void run() {
        System.out.println("[Player]run()");
        while (true) {
            try {
                Thread.sleep(1000000);
            } catch (InterruptedException ex) {
            }
        }
    }

    // kreiere layer und starte ihn
    public static void main(String args[]) {
        System.out.println("[Player]main()");
        if (args.length < 2) {
            System.out.println("Usage: Player gameName playerName");
            System.exit(1);
        }

        try {
            Player player = new Player(args[0], args[1]);
            new Thread(player).start();
        } catch (IOException e) {
            System.out.println("Player konnte nicht kreiert werden: " +
                e.getMessage());
        }
    }
}